/* *********************************** */
#include "tac_plus.h"
#include "md5.h"

#include <signal.h>
#include <pwd.h>
#include <stdarg.h>
/***************************************/


struct session session;


/***************** AUTHENTICATION ********************/

#define TACACS_ENABLE_REQUEST  1
#define TACACS_ASCII_LOGIN     2
#define TACACS_PAP_LOGIN       3
#define TACACS_CHAP_LOGIN      4
#define TACACS_ARAP_LOGIN      5
#define TACACS_PAP_OUT         6
#define TACACS_CHAP_OUT        7
#define TACACS_ASCII_CHPASS    9
#define TACACS_MSCHAP_LOGIN    12
#define TACACS_MSCHAP_OUT      13
/*    
   (     ) */
/* port - tty10  Async10    
   username -  

type:
   1  - Enable Requests
   2  - Inbound ASCII Login
   3  - Inbound PAP Login
   4  - Inbound CHAP login
   5  - Inbound ARAP login
   6  - Outbound PAP request
   7  - Outbound CHAP request
   8  - Outbound ASCII and ARAP request
   9  - ASCII change password request
   10 - PPP change password request
   11 - ARAP change password request
   12 - MS-CHAP inbound login
   13 - MS-CHAP outbound login
*/
int send_authen_start(port,username,type,data)
char *port;
char *username;
int type;
char *data;
{
  char buf[256];
  char name[70];
  char addr[57];
  /*  */
  HDR *hdr = (HDR *)buf;
  /*  */
  struct authen_start *ask=(struct authen_start *)(buf+TAC_PLUS_HDR_SIZE);
  /*   */
  /*  */
  char *u=(char *)(buf+TAC_PLUS_HDR_SIZE+TAC_AUTHEN_START_FIXED_FIELDS_SIZE);
  /*  */
  char *p=(char *)(u+strlen(username));
  /*  */
  char *a=(char *)(p+strlen(port));
  /*  */
  char *d;
  
  /*  */
  bzero(buf, sizeof(buf));
  /*   */
  gethostname(name,49);
  strcpy(addr,getipfromname(name));

  d = (char *)(a + strlen(addr));

  /***  ***/
  /*  (   TAC_PLUS_MAJOR_VER | TAC_PLUS_MINOR_VER_0) */
  if  (type == TACACS_ENABLE_REQUEST ||
       type == TACACS_ASCII_LOGIN )
	   hdr->version = TAC_PLUS_VER_0;
  else
	   hdr->version = TAC_PLUS_VER_1;
  /*   - TAC_PLUS_AUTHEN */
  hdr->type = TAC_PLUS_AUTHEN;
  /*  sequence,        1 */
  hdr->seq_no = ++session.seq_no;
  /*  TAC_PLUS_ENCRYPTED || TAC_PLUS_CLEAR */
  hdr->encryption = TAC_PLUS_ENCRYPTED;
  /* id  */
  hdr->session_id = session.session_id;

  /*   */
  if (type == TACACS_CHAP_LOGIN || type == TACACS_MSCHAP_LOGIN)
      hdr->datalength = htonl(TAC_AUTHEN_START_FIXED_FIELDS_SIZE
	   +strlen(username)+strlen(port)+strlen(addr)+1+strlen(data));
  else
  if (type == TACACS_PAP_LOGIN || type == TACACS_ARAP_LOGIN)
      hdr->datalength = htonl(TAC_AUTHEN_START_FIXED_FIELDS_SIZE
	   +strlen(username)+strlen(port)+strlen(addr)+strlen(data));
  else
      hdr->datalength = htonl(TAC_AUTHEN_START_FIXED_FIELDS_SIZE
	   +strlen(username)+strlen(port)+strlen(addr));

  /***  ***/
  /*   */
  ask->priv_lvl = TAC_PLUS_PRIV_LVL_MIN;
  switch (type)
  {
    case TACACS_ENABLE_REQUEST:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->service = TAC_PLUS_AUTHEN_SVC_ENABLE;
       break;
    case TACACS_ASCII_LOGIN:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_ASCII;
       ask->service = TAC_PLUS_AUTHEN_SVC_LOGIN;
       break;
    case TACACS_PAP_LOGIN:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_PAP;
       break;
    case TACACS_PAP_OUT:
       ask->action = TAC_PLUS_AUTHEN_SENDAUTH;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_PAP;
       break;
    case TACACS_CHAP_LOGIN:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_CHAP;
       break;
    case TACACS_CHAP_OUT:
       ask->action = TAC_PLUS_AUTHEN_SENDAUTH;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_CHAP;
       break;
    case TACACS_MSCHAP_LOGIN:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_MSCHAP;
       break;
    case TACACS_MSCHAP_OUT:
       ask->action = TAC_PLUS_AUTHEN_SENDAUTH;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_MSCHAP;
       break;
    case TACACS_ARAP_LOGIN:
       ask->action = TAC_PLUS_AUTHEN_LOGIN;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_ARAP;
       break;
    case TACACS_ASCII_CHPASS:
       ask->action = TAC_PLUS_AUTHEN_CHPASS;
       ask->authen_type = TAC_PLUS_AUTHEN_TYPE_ASCII;
       break;
  }
  /*
   *      
   *  ,    ntohs  htons
   */
  /*    */
  ask->user_len = strlen(username);
  /*     ( tty10  ) */
  ask->port_len = strlen(port);
  /*   */
  ask->rem_addr_len = strlen(addr);
  /*   */
  ask->data_len = strlen(data);

  /*    */
  if (strlen(username) > 0)
    strcpy(u, username); /*  */
  if (strlen(port) > 0)
    strcpy(p, port);     /*  */
  if (strlen(addr) > 0)
    strcpy(a, addr);     /*  */
  if (type == TACACS_CHAP_LOGIN)
  {
     *d++ = 1;
     strcpy(d,data);
  }
  if (type == TACACS_ARAP_LOGIN || type == TACACS_PAP_LOGIN)
     strcpy(d,data);

  /* write_packet   */
  if (write_packet(buf)) return 1;
}


/**********************************************
 *  get_authen_start  ( )
 *    
 *  return 0 - error, type - success
 */
int get_authen_start(pak,username,port,rem_addr,data)
char *pak;
char *username;
char *port;
char *rem_addr;
char *data;
{
   char msg[55];
   struct authen_start *start=(struct authen_start *)(pak+TAC_PLUS_HDR_SIZE);
   HDR *hdr = (HDR *) pak;
   int type;
   char *u = (char *)
	(pak+TAC_PLUS_HDR_SIZE+TAC_AUTHEN_START_FIXED_FIELDS_SIZE);
   char *p = (char *)(u + start->user_len);
   char *r = (char *)(p + start->port_len);
   char *d = (char *)(r + start->rem_addr_len);

   session.session_id = ntohl(hdr->session_id);
   strcpy(username,"");
   strcpy(port,"");
   strcpy(rem_addr,"");
   strcpy(data,"");
   if (hdr->seq_no != 1) {
       printf("Invalid sequence");
       return 0;
   }
   if (start->action != TAC_PLUS_AUTHEN_LOGIN) {
       printf("Invalid AUTHEN/START action=%d", start->action);
       return 0;
   }
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->service == TAC_PLUS_AUTHEN_SVC_ENABLE)
       type = TACACS_ENABLE_REQUEST;
   if (start->action == TAC_PLUS_AUTHEN_CHPASS &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_ASCII)
       type = TACACS_ASCII_CHPASS;
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->service == TAC_PLUS_AUTHEN_SVC_LOGIN &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_ASCII)
       type = TACACS_ASCII_LOGIN;
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_PAP)
       type = TACACS_PAP_LOGIN;
   if (start->action == TAC_PLUS_AUTHEN_SENDAUTH &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_PAP)
       type = TACACS_PAP_OUT;
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_CHAP)
       type = TACACS_CHAP_LOGIN;
   if (start->action == TAC_PLUS_AUTHEN_SENDAUTH &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_CHAP)
       type = TACACS_CHAP_OUT;
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_MSCHAP)
       type = TACACS_MSCHAP_LOGIN;
   if (start->action == TAC_PLUS_AUTHEN_SENDAUTH &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_MSCHAP)
       type = TACACS_MSCHAP_OUT;
   if (start->action == TAC_PLUS_AUTHEN_LOGIN &&
       start->authen_type == TAC_PLUS_AUTHEN_TYPE_ARAP)
       type = TACACS_ARAP_LOGIN;

   /*  */
   strncpy(username,u,start->user_len);
   /*  */
   username[start->user_len+1]=0;
   /*  */
   strncpy(port,p,start->port_len);
   /*  */
   port[start->port_len+1]=0;
   /*  */
   strncpy(rem_addr,r,start->rem_addr_len);
   /*  */
   rem_addr[start->rem_addr_len+1]=0;
   /*  */
   strncpy(data,d,start->data_len);
   /*  */
   data[start->data_len+1]=0;

   return (type);
}

/*********************************************
    REPLY ( )
     
     
 :

   TAC_PLUS_AUTHEN_STATUS_PASS     := 0x01
   TAC_PLUS_AUTHEN_STATUS_FAIL     := 0x02
   TAC_PLUS_AUTHEN_STATUS_GETDATA  := 0x03
   TAC_PLUS_AUTHEN_STATUS_GETUSER  := 0x04
   TAC_PLUS_AUTHEN_STATUS_GETPASS  := 0x05
   TAC_PLUS_AUTHEN_STATUS_RESTART  := 0x06
   TAC_PLUS_AUTHEN_STATUS_ERROR    := 0x07
   TAC_PLUS_AUTHEN_STATUS_FOLLOW   := 0x21
***********************************************/
int send_reply_packet(status,server_msg,data)
int status;
char *server_msg;
char *data;
{
   char buf[256];
   /*  */
   HDR *hdr = (HDR *)buf;
   /*  */
   struct authen_reply *dat=(struct authen_reply *)(buf+TAC_PLUS_HDR_SIZE);
   char *s=(char *)
	   (buf+TAC_PLUS_HDR_SIZE+TAC_AUTHEN_REPLY_FIXED_FIELDS_SIZE);
   char *d=(char *)(s+strlen(server_msg));

   /*  */
   bzero(buf, sizeof(buf));
   /*  */
   hdr->version = TAC_PLUS_VER_0;
   /*   */
   hdr->type = TAC_PLUS_AUTHEN;
   /* sequence  */
   hdr->seq_no = ++session.seq_no;
   /*   */
   hdr->encryption = TAC_PLUS_ENCRYPTED; /* TAC_PLUS_CLEAR; */
   /* id  */
   hdr->session_id = session.session_id;
   /*   */
   hdr->datalength = htonl(TAC_AUTHEN_REPLY_FIXED_FIELDS_SIZE
		     +strlen(server_msg)+strlen(data));

   /*    */
   dat->status = status;
   dat->flags = 1;
   dat->msg_len = htons(strlen(server_msg));
   dat->data_len = htons(strlen(data));

   strcpy(s,server_msg);
   strcpy(d,data);

   if (write_packet(buf)) return 1;
}

/*  REPLY  () */
/*   
     
 :

   TAC_PLUS_AUTHEN_STATUS_PASS     := 0x01
   TAC_PLUS_AUTHEN_STATUS_FAIL     := 0x02
   TAC_PLUS_AUTHEN_STATUS_GETDATA  := 0x03
   TAC_PLUS_AUTHEN_STATUS_GETUSER  := 0x04
   TAC_PLUS_AUTHEN_STATUS_GETPASS  := 0x05
   TAC_PLUS_AUTHEN_STATUS_RESTART  := 0x06
   TAC_PLUS_AUTHEN_STATUS_ERROR    := 0x07
   TAC_PLUS_AUTHEN_STATUS_FOLLOW   := 0x21

   0,  ...
*/
int get_reply_packet(server,datas)
char *server;
char *datas;
{
   /*  */
   char *buf = read_packet();
   /*  */
   HDR *hdr = (HDR *)buf;
   /*   */
   struct authen_reply *rep = (struct authen_reply *)(buf+TAC_PLUS_HDR_SIZE);
   /*     */
   char *serv_msg = (char *)
	(buf+TAC_PLUS_HDR_SIZE+TAC_AUTHEN_REPLY_FIXED_FIELDS_SIZE);
   /*     */
   char *dat_pak=(char *)(serv_msg + ntohs(rep->msg_len));
   int mlen=0,dlen=0;

   strcpy(server,"");
   strcpy(datas,"");
   /*   */
   mlen = ntohs(rep->msg_len);
   dlen = ntohs(rep->data_len);
   printf("mlen=%d, dlen=%d\n",mlen,dlen);

   if (hdr->datalength != htonl(TAC_AUTHEN_REPLY_FIXED_FIELDS_SIZE +
	       mlen + dlen))
   {
       printf("Invalid AUTHEN/REPLY packet! Check keys...\n");
       return 0;
   }

   session.session_id = ntohl(hdr->session_id);
printf("Point 1\n");
   if (mlen > 0)
   {
printf("Point 1.2, mlen=%d\n",mlen);
      /*     */
      strncpy(server,serv_msg,mlen);
      /*      */
      server[mlen+1]=0;
   }
printf("Point 2\n");
   if (dlen > 0)
   {
      strncpy(datas,dat_pak,dlen);
      /*    */
      datas[dlen+1]=0;
   }
   return (rep->status);
}

/*  CONTINUE  */
/* ( ) */
int send_authen_cont(user_msg,data)
char *user_msg;
char *data;
{
  int len;
  char buf[256];
  /*  */
  HDR *hdr = (HDR *)buf;
  /*  */
  struct authen_cont *ask = (struct authen_cont *)(buf + TAC_PLUS_HDR_SIZE);
  /*   */
  char *p = (char *)
    (buf + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE);
  char *d = (char *)(p + sizeof(user_msg));
  
  /*  */
  bzero(buf, sizeof(buf));

  /*  */
  hdr->version = TAC_PLUS_VER_0;
  /*   */
  hdr->type = TAC_PLUS_AUTHEN;
  /* sequence  */
  hdr->seq_no = ++session.seq_no;
  /*   */
  hdr->encryption = TAC_PLUS_ENCRYPTED; /* TAC_PLUS_CLEAR; */
  /* id  */
  hdr->session_id = session.session_id;
  /*   */
  hdr->datalength = htonl(TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE+strlen(user_msg)
			   + strlen(data));

  /*   */
  ask->user_msg_len = htons(strlen(user_msg));
  ask->user_data_len = htons(strlen(data));

  /*    */
  if (strlen(user_msg) > 0)
    strcpy(p, user_msg);
  if (strlen(data) > 0)
    strcpy(d, data);

  /*   (  ) */
  if (write_packet(buf)) return 1;
}

/******
  get_authen_cont ()
   CONTINUE 
*/
int get_authen_cont(user_msg,data)
char *user_msg;
char *data;
{
   /*  */
   char *buf = read_packet();
   HDR *hdr = (HDR *)buf;
   /*  */
   struct authen_cont *ask = (struct authen_cont *)(buf + TAC_PLUS_HDR_SIZE);
   /*   */
   char *p = (char *)
     (buf + TAC_PLUS_HDR_SIZE + TAC_AUTHEN_CONT_FIXED_FIELDS_SIZE);
   char *d = (char *)(p + ntohs(ask->user_msg_len));

   if (hdr->datalength != ntohl(TAC_AUTHEN_REPLY_FIXED_FIELDS_SIZE +
	      ntohs(ask->user_msg_len) + ntohs(ask->user_data_len)))
   {
       printf("Invalid AUTHEN/CONT packet! Check keys...\n");
       return 0;
   }

   session.session_id = ntohl(hdr->session_id);
   strcpy(user_msg,"");
   strcpy(data,"");

   strncpy(user_msg,p,ntohs(ask->user_msg_len));
   /*     */
   user_msg[ntohs(ask->user_msg_len)+1]=0;
   strncpy(data,d,ntohs(ask->user_data_len));
   /*   */
   data[ntohs(ask->user_data_len)+1]=0;

   return 1;
}

/************************************************************/

