/* *********************************** */

#include "tac_plus.h"
#include "md5.h"

#include <signal.h>
#include <pwd.h>
#include <stdarg.h>

#define tac_abort { session.aborted = 1; return 0; }

/*    xxx.xxx.xxx.xxx   name.domain.ru */
char *getipfromname(name)
char *name;
{
   struct    in_addr  nas_addr;
   struct hostent *host;
   char hostaddr[40];

   host = gethostbyname(name);
   if (host == NULL) {
	printf("gethostbyname(%s) failure\n",name);
   }
   bcopy(host->h_addr, (char *)&nas_addr, host->h_length);
   strcpy(hostaddr,inet_ntoa(nas_addr));

   return (hostaddr);
}


/*   timeout   */
static void catchup(s)
{
   printf("\r\n*** ERROR: TACACS+ SERVER NOT RESPONDING!\r\n");
   session.aborted = 1;
   return;
}


/***********************************/
/*******    *********/
/***********************************/
/*  ( ) */
/*  0 - , 1-  */
int tac_connect(char *peer,int timeout,char *key,int port)
{
  int f;
  struct sockaddr_in s;
  void (*oldalrm)();
  char buf[256];

  /*      */
  if (session.peer) free(session.peer);
  if (session.key) free(session.key);
  /*     */
  session.peer = strdup(peer);
  session.key = strdup(key);
  session.aborted = 0;

  /*   */
  if ((f = socket(AF_INET, SOCK_STREAM, 0)) < 0) tac_abort;
  s.sin_addr.s_addr = htonl(INADDR_ANY);
#ifndef __linux__
  s.sin_len = sizeof(struct sockaddr_in);
#endif
  s.sin_family = AF_INET;
  s.sin_port = 0;
  if (bind(f, (struct sockaddr *)&s, sizeof(s)) < 0) tac_abort;
  if (!inet_aton(session.peer, &s.sin_addr.s_addr)) tac_abort;
#ifndef __linux__
  s.sin_len = sizeof(struct sockaddr_in);
#endif
  s.sin_family = AF_INET;
  s.sin_port = htons(port);
  oldalrm = signal(SIGALRM, catchup);
  alarm(timeout);
  if (connect(f, (struct sockaddr *)&s, sizeof(s)) < 0) tac_abort;
  alarm(0);
  session.sock = f;
  /*  session_id  pid  */
  session.session_id = htonl(getpid());
  /* sequence  */
  session.seq_no = 0;
  /*   ? */
  session.last_exch = time(NULL);
  signal(SIGALRM, oldalrm);

  return session.aborted;
}



/**********************************************************/

/*
 * create_md5_hash(): create an md5 hash of the "session_id", "the user's
 * key", "the version number", the "sequence number", and an optional
 * 16 bytes of data (a previously calculated hash). If not present, this
 * should be NULL pointer.
 *
 * Write resulting hash into the array pointed to by "hash".
 *
 * The caller must allocate sufficient space for the resulting hash
 * (which is 16 bytes long). The resulting hash can safely be used as
 * input to another call to create_md5_hash, as its contents are copied
 * before the new hash is generated.
 */
void
create_md5_hash(session_id, key, version, seq_no, prev_hash, hash)
int session_id;
char *key;
u_char version;
u_char seq_no;
u_char *prev_hash;
u_char *hash;
{
    u_char *md_stream, *mdp;
    int md_len;
    MD5_CTX mdcontext;

    md_len = sizeof(session_id) + strlen(key) + sizeof(version) +
	sizeof(seq_no);

    if (prev_hash) {
	md_len += MD5_LEN;
    }
    mdp = md_stream = (u_char *) malloc(md_len);
    bcopy(&session_id, mdp, sizeof(session_id));
    mdp += sizeof(session_id);

    bcopy(key, mdp, strlen(key));
    mdp += strlen(key);

    bcopy(&version, mdp, sizeof(version));
    mdp += sizeof(version);

    bcopy(&seq_no, mdp, sizeof(seq_no));
    mdp += sizeof(seq_no);

    if (prev_hash) {
	bcopy(prev_hash, mdp, MD5_LEN);
	mdp += MD5_LEN;
    }
    MD5Init(&mdcontext);
    MD5Update(&mdcontext, md_stream, md_len);
    MD5Final(hash, &mdcontext);
    free(md_stream);
    return;
}

/*
 * Overwrite input data with en/decrypted version by generating an MD5 hash and
 * xor'ing data with it.
 *
 * When more than 16 bytes of hash is needed, the MD5 hash is performed
 * again with the same values as before, but with the previous hash value
 * appended to the MD5 input stream.
 *
 * Return 0 on success, -1 on failure.
 */
md5_xor(hdr, data, key)
HDR *hdr;
u_char *data;
char *key;
{
    int i, j;
    u_char hash[MD5_LEN];       /* the md5 hash */
    u_char last_hash[MD5_LEN];  /* the last hash we generated */
    u_char *prev_hashp = (u_char *) NULL;       /* pointer to last created
						 * hash */
    int data_len;
    int session_id;
    u_char version;
    u_char seq_no;

    data_len = ntohl(hdr->datalength);
    session_id = hdr->session_id; /* always in network order for hashing */
    version = hdr->version;
    seq_no = hdr->seq_no;

    if (!key)
	return (0);

    for (i = 0; i < data_len; i += 16) {

	create_md5_hash(session_id, key, version, seq_no, prev_hashp, hash);

	if (DEBUG_MD5_HASH_FLAG) {
	    int k;

	 /* printf(
		   "hash: session_id=%u, key=%s, version=%d, seq_no=%d\n",
		   session_id, key, version, seq_no); */
	}                       /* debug */
	bcopy(hash, last_hash, MD5_LEN);
	prev_hashp = last_hash;

	for (j = 0; j < 16; j++) {

	    if ((i + j) >= data_len) {
		hdr->encryption = (hdr->encryption == TAC_PLUS_CLEAR)
		    ? TAC_PLUS_ENCRYPTED : TAC_PLUS_CLEAR;
		return (0);
	    }
	    data[i + j] ^= hash[j];
	}
    }
    hdr->encryption = (hdr->encryption == TAC_PLUS_CLEAR)
	? TAC_PLUS_ENCRYPTED : TAC_PLUS_CLEAR;
    return (0);
}
/*********************************************************/


/*  n    fd   ptr   t
 * .     ,  
 *   .
 *
 *  -1  , eof  .  
 *   . */

int
sockread(fd, ptr, nbytes, timeout)
int fd;
u_char *ptr;
int nbytes;
int timeout;
{
    int nleft, nread;
    fd_set readfds, exceptfds;
    struct timeval tout;

    tout.tv_sec = timeout;
    tout.tv_usec = 0;

    FD_ZERO(&readfds);
    FD_SET(fd, &readfds);

    FD_ZERO(&exceptfds);
    FD_SET(fd, &exceptfds);

    nleft = nbytes;

    while (nleft > 0) {
	int status = select(fd + 1, &readfds, (fd_set *) NULL,
			    &exceptfds, &tout);

	if (status == 0) {
	    printf("%s: timeout reading fd %d", session.peer, fd);
	    return(-1);
	}
	if (status < 0) {
	    if (errno == EINTR)
		continue;
	    printf("%s: error in select %s fd %d",
		   session.peer, sys_errlist[errno], fd);
	    return (-1);
	}
	if (FD_ISSET(fd, &exceptfds)) {
	    printf("%s: exception on fd %d",
		   session.peer, fd);
	    return (-1);
	}
	if (!FD_ISSET(fd, &readfds)) {
	    printf("%s: spurious return from select",
		   session.peer);
	    continue;
	}
    again:
	nread = read(fd, ptr, nleft);

	if (nread < 0) {
	    if (errno == EINTR)
		goto again;
	    printf("%s %s: error reading fd %d nread=%d %s",
		   session.peer, session.port, fd, nread, sys_errlist[errno]);
	    return (-1);        /* error */

	} else if (nread == 0) {
	    printf("%s %s: fd %d eof (connection closed)",
		   session.peer, session.port, fd);
	    return (-1);        /* eof */
	}
	nleft -= nread;
	if (nleft)
	    ptr += nread;
    }
    return (nbytes - nleft);
}


/* Write n bytes to descriptor fd from array ptr with timeout t
 * seconds. Note the timeout is applied to each write, not for the
 * overall operation.
 *
 * Return -1 on error, eof or timeout. Otherwise return number of
 * bytes written. */

int
sockwrite(fd, ptr, bytes, timeout)
int fd;
u_char *ptr;
int bytes;
int timeout;
{
    int remaining, sent;
    fd_set writefds, exceptfds;
    struct timeval tout;

    sent = 0;

    tout.tv_sec = timeout;
    tout.tv_usec = 0;


    FD_ZERO(&writefds);
    FD_SET(fd, &writefds);

    FD_ZERO(&exceptfds);
    FD_SET(fd, &exceptfds);

    remaining = bytes;

    while (remaining > 0) {
	int status = select(fd + 1, (fd_set *) NULL,
			    &writefds, &exceptfds, &tout);

	if (status == 0) {
	    printf("%s: timeout writing to fd %d",
		   session.peer, fd);
	    return (-1);
	}
	if (status < 0) {
	    printf("%s: error in select fd %d",
		   session.peer, fd);
	    return (-1);
	}
	if (FD_ISSET(fd, &exceptfds)) {
	    printf("%s: exception on fd %d",
		   session.peer, fd);
	    return (sent);      /* error */
	}

	if (!FD_ISSET(fd, &writefds)) {
	    printf("%s: spurious return from select",
		   session.peer);
	    continue;
	}
	sent = write(fd, ptr, remaining);

	if (sent <= 0) {
	    printf("%s: error writing fd %d sent=%d",
		   session.peer, fd, sent);
	    return (sent);      /* error */
	}
	remaining -= sent;
	ptr += sent;
    }
    return (bytes - remaining);
}

/*      . 
 seq_no,  NULL   */

u_char *
read_packet()
{
    HDR hdr;
    u_char *pkt, *data;
    int len;

    /* read a packet header */
    len = sockread(session.sock, (u_char *) & hdr, TAC_PLUS_HDR_SIZE, TAC_PLUS_READ_TIMEOUT);
    if (len != TAC_PLUS_HDR_SIZE) {
	printf("Read %d bytes from %s %s, expecting %d",
	       len, session.peer, session.port, TAC_PLUS_HDR_SIZE);
	return(NULL);
    }

    if ((hdr.version & TAC_PLUS_MAJOR_VER_MASK) != TAC_PLUS_MAJOR_VER) {
	printf(
	       "%s: Illegal major version specified: found %d wanted %d\n",
	       session.peer, hdr.version, TAC_PLUS_MAJOR_VER);
	return(NULL);
    }

    /* get memory for the packet */
    len = TAC_PLUS_HDR_SIZE + ntohl(hdr.datalength);
    pkt = (u_char *) malloc(len);

    /* initialise the packet */
    bcopy(&hdr, pkt, TAC_PLUS_HDR_SIZE);

    /* the data start here */
    data = pkt + TAC_PLUS_HDR_SIZE;

    /* read the rest of the packet data */
    if (sockread(session.sock, data, ntohl(hdr.datalength),
		 TAC_PLUS_READ_TIMEOUT) !=
	ntohl(hdr.datalength)) {
	printf("%s: start_session: bad socket read", session.peer);
	return (NULL);
    }
    session.seq_no++;           /* should now equal that of incoming packet */
    session.last_exch = time(NULL);

    if (session.seq_no != hdr.seq_no) {
	printf("%s: Illegal session seq # %d != packet seq # %d",
	       session.peer,
	       session.seq_no, hdr.seq_no);
	return (NULL);
    }

    /* decrypt the data portion */
    if (md5_xor((HDR *)pkt, data, session.key)) {
	printf("%s: start_session error decrypting data",
	       session.peer);
	return (NULL);
    }

    session.version = hdr.version;

    return (pkt);
}

/* write a packet to the wire, encrypting it */
write_packet(pak)
u_char *pak;
{
    HDR *hdr = (HDR *) pak;
    u_char *data;
    int len;

    len = TAC_PLUS_HDR_SIZE + ntohl(hdr->datalength);

    /* the data start here */
    data = pak + TAC_PLUS_HDR_SIZE;

    /* encrypt the data portion */
    if (md5_xor((HDR *)pak, data, session.key)) {
	printf("%s: write_packet: error encrypting data", session.peer);
	return (-1);
    }

    if (sockwrite(session.sock, pak, len, TAC_PLUS_WRITE_TIMEOUT) != len) {
	return (-1);
    }
    session.last_exch = time(NULL);
    return (0);
}

/*************************************************/
